(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "../isNode", "../TypeGuards/isUUIDVersion", "./makeVersionFourOrNilUUIDValues", "./makeVersionOneUUIDValues", "./makeVersionThreeOrFiveUUIDValues", "./UUIDOptions/mergeUUIDOptions", "../strings", "../uintArrayAsHex", "../uintArrayAsBigNumber", "./UUIDOptions/UUIDOptions", "../Enums/UUIDVersions", "../writeNewResults"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const isNode_1 = require("../isNode");
    const isUUIDVersion_1 = require("../TypeGuards/isUUIDVersion");
    const makeVersionFourOrNilUUIDValues_1 = require("./makeVersionFourOrNilUUIDValues");
    const makeVersionOneUUIDValues_1 = require("./makeVersionOneUUIDValues");
    const makeVersionThreeOrFiveUUIDValues_1 = require("./makeVersionThreeOrFiveUUIDValues");
    const mergeUUIDOptions_1 = require("./UUIDOptions/mergeUUIDOptions");
    const strings_1 = require("../strings");
    const uintArrayAsHex_1 = require("../uintArrayAsHex");
    const uintArrayAsBigNumber_1 = require("../uintArrayAsBigNumber");
    const UUIDOptions_1 = require("./UUIDOptions/UUIDOptions");
    const UUIDVersions_1 = require("../Enums/UUIDVersions");
    const writeNewResults_1 = require("../writeNewResults");
    class UUID {
        constructor(argOptions) {
            const base = new UUIDOptions_1.UUIDOptions();
            const options = argOptions ? mergeUUIDOptions_1.mergeUUIDOptions(base, argOptions) : base;
            let version = options.version.toString();
            if (!isUUIDVersion_1.isUUIDVersion(version)) {
                throw new Error(strings_1.strings.UUID_VERSION_INVALID);
            }
            if (!isNode_1.isNode() && version === UUIDVersions_1.UUIDVersions.One) {
                throw new Error(strings_1.strings.VERSION_1_IN_BROWSER);
            }
            this.__version = version;
            const { clockSequence, nodeIdentifier, shouldWrite, timestamp, } = (() => {
                if (version === UUIDVersions_1.UUIDVersions.One) {
                    return makeVersionOneUUIDValues_1.makeVersionOneUUIDValues(options);
                }
                else if (version === UUIDVersions_1.UUIDVersions.Three ||
                    version === UUIDVersions_1.UUIDVersions.Five) {
                    return makeVersionThreeOrFiveUUIDValues_1.makeVersionThreeOrFiveUUIDValues(options);
                }
                else {
                    return makeVersionFourOrNilUUIDValues_1.makeVersionFourOrNilUUIDValues(options);
                }
            })();
            this.__clockSequence = clockSequence;
            this.__nodeIdentifier = nodeIdentifier;
            this.__timestamp = timestamp;
            if (shouldWrite) {
                writeNewResults_1.writeNewResults(this);
            }
        }
        get id() {
            return this.toString();
        }
        get version() {
            return this.__version;
        }
        get timestamp() {
            return this.__timestamp;
        }
        get timeLow() {
            return this.timestamp.slice(4, 8);
        }
        get timeMid() {
            return this.timestamp.slice(2, 4);
        }
        get timeHigh() {
            return this.timestamp.slice(0, 2);
        }
        get clockSequence() {
            return this.__clockSequence;
        }
        get clockSequenceLow() {
            return this.clockSequence.slice(1);
        }
        get clockSequenceHigh() {
            return this.clockSequence.slice(0, 1);
        }
        get reserved() {
            return new Uint8Array([2,]);
        }
        get clockSequenceHighAndReserved() {
            const clockHigh = uintArrayAsBigNumber_1.uintArrayAsBigNumber(this.clockSequenceHigh).toString(2);
            const reserved = this.version === UUIDVersions_1.UUIDVersions.Nil ?
                '0' :
                uintArrayAsBigNumber_1.uintArrayAsBigNumber(this.reserved).toString(2);
            const byte = clockHigh.padStart(6, '0') + reserved.padStart(2, '0');
            return new Uint8Array([parseInt(byte, 2),]);
        }
        get nodeIdentifier() {
            return this.__nodeIdentifier;
        }
        static parse(text) {
            const split = text.split('-');
            if (split.length !== 5) {
                throw new Error(strings_1.strings.UUID_STRING_INVALID);
            }
            const timeLow = split[0];
            const timeMid = split[1];
            const timeHighAndVersion = split[2];
            const timeHigh = timeHighAndVersion.slice(0, 3);
            const version = timeHighAndVersion[0];
            const timestampHex = timeHigh + timeMid + timeLow;
            const timestampArr = [];
            for (let ii = 0; ii < 16; ii += 2) {
                timestampArr.push(parseInt(timestampHex.slice(ii, ii + 2), 16));
            }
            const timestamp = new Uint8Array(timestampArr);
            const clockSequenceHighAndReservedAndLow = split[3];
            const clockSequenceHighAndReservedHex = clockSequenceHighAndReservedAndLow.slice(0, 2);
            const clockSequenceHighHex = parseInt(parseInt(clockSequenceHighAndReservedHex, 16).toString(2).slice(2), 2).toString(16);
            const clockSequenceLowHex = clockSequenceHighAndReservedAndLow.slice(2, 4);
            const clockSequenceHex = clockSequenceLowHex + clockSequenceHighHex;
            const clockSequenceArr = [];
            for (let ii = 0; ii < 7; ii += 2) {
                clockSequenceArr.push(parseInt(clockSequenceHex.slice(ii, ii + 2), 16));
            }
            const clockSequence = new Uint8Array(clockSequenceArr);
            const nodeIdentifierHex = split[4];
            const nodeIdentifierArr = [];
            for (let ii = 0; ii < 12; ii += 2) {
                nodeIdentifierArr.push(parseInt(nodeIdentifierHex.slice(ii, ii + 2), 16));
            }
            const nodeIdentifier = new Uint8Array(nodeIdentifierArr);
            const obj = {
                __version: version,
                get version() {
                    return obj.__version;
                },
                __timestamp: timestamp,
                get timestamp() {
                    return obj.__timestamp;
                },
                __clockSequence: clockSequence,
                get clockSequence() {
                    return obj.__clockSequence;
                },
                __nodeIdentifier: nodeIdentifier,
                get nodeIdentifier() {
                    return obj.__nodeIdentifier;
                },
            };
            return Object.assign({}, this.prototype, obj);
        }
        toString() {
            const format = (value, toPad) => (uintArrayAsHex_1.uintArrayAsHex(value)
                .padStart(toPad, '0'));
            return (format(this.timeLow, 8) +
                '-' +
                format(this.timeMid, 4) +
                '-' +
                `${this.version}${format(this.timeHigh, 3)}` +
                '-' +
                `${format(this.clockSequenceHighAndReserved, 2)}${format(this.clockSequenceLow, 2)}` +
                '-' +
                format(this.nodeIdentifier, 12));
        }
    }
    exports.UUID = UUID;
    exports.default = UUID;
});
