(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "../../clockSequenceGetter", "../../TypeGuards/isUUIDVersion", "../../nodeIdentifierGetter", "../../strings", "../../timestampGetter", "../../Enums/UUIDVersions"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const clockSequenceGetter_1 = require("../../clockSequenceGetter");
    const isUUIDVersion_1 = require("../../TypeGuards/isUUIDVersion");
    const nodeIdentifierGetter_1 = require("../../nodeIdentifierGetter");
    const strings_1 = require("../../strings");
    const timestampGetter_1 = require("../../timestampGetter");
    const UUIDVersions_1 = require("../../Enums/UUIDVersions");
    class UUIDOptions {
        constructor(_args = {}) {
            this.version = UUIDVersions_1.UUIDVersions.Four;
            this.clockSequenceGetter = clockSequenceGetter_1.clockSequenceGetter;
            this.nodeIdentifierGetter = nodeIdentifierGetter_1.nodeIdentifierGetter;
            this.timestampGetter = timestampGetter_1.timestampGetter;
            const args = _args || {};
            if (args.version) {
                let ver = args.version.toString();
                if (!isUUIDVersion_1.isUUIDVersion(ver)) {
                    throw new Error(strings_1.strings.UUID_VERSION_INVALID);
                }
                this.version = ver;
            }
            if (typeof args.clockSequenceGetter === 'function') {
                this.clockSequenceGetter = args.clockSequenceGetter;
            }
            if (typeof args.nodeIdentifierGetter === 'function') {
                this.nodeIdentifierGetter = args.nodeIdentifierGetter;
            }
            if (typeof args.timestampGetter === 'function') {
                this.timestampGetter = args.timestampGetter;
            }
            if (this.version === UUIDVersions_1.UUIDVersions.Three ||
                this.version === UUIDVersions_1.UUIDVersions.Five) {
                if (args.namespaceId) {
                    this.namespaceId = args.namespaceId;
                }
                else {
                    throw new Error(strings_1.strings.NAMESPACE_ID_MISSING);
                }
                if (args.name) {
                    this.name = args.name;
                }
                else {
                    throw new Error(strings_1.strings.NAME_MISSING);
                }
            }
        }
    }
    exports.UUIDOptions = UUIDOptions;
    exports.default = UUIDOptions;
});
