(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./convertBinStrToUint8Array", "./TypeGuards/isUUIDVersion", "./getLastResults", "./randomBytesGenerator", "./strings", "./uintArrayAsBigNumber", "./Enums/UUIDVersions"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const convertBinStrToUint8Array_1 = require("./convertBinStrToUint8Array");
    const isUUIDVersion_1 = require("./TypeGuards/isUUIDVersion");
    const getLastResults_1 = require("./getLastResults");
    const randomBytesGenerator_1 = require("./randomBytesGenerator");
    const strings_1 = require("./strings");
    const uintArrayAsBigNumber_1 = require("./uintArrayAsBigNumber");
    const UUIDVersions_1 = require("./Enums/UUIDVersions");
    exports.clockSequenceGetter = (version, hash) => {
        if (!isUUIDVersion_1.isUUIDVersion(version)) {
            throw new Error(strings_1.strings.UUID_VERSION_INVALID);
        }
        let clockSequence;
        if (version === UUIDVersions_1.UUIDVersions.One || version === UUIDVersions_1.UUIDVersions.Four) {
            const getRandomSeq = () => {
                const clockSequenceNum = uintArrayAsBigNumber_1.uintArrayAsBigNumber(randomBytesGenerator_1.randomBytesGenerator(2));
                const clockSequenceBin = clockSequenceNum.toString(2).slice(0, 14);
                return new Uint8Array([
                    parseInt(clockSequenceBin.slice(0, 6), 2),
                    parseInt(clockSequenceBin.slice(6), 2),
                ]);
            };
            const lastResults = version === UUIDVersions_1.UUIDVersions.One ? getLastResults_1.getLastResults() : null;
            if (lastResults &&
                lastResults.clockSequence &&
                'BYTES_PER_ELEMENT' in lastResults.clockSequence) {
                return lastResults.clockSequence;
            }
            else {
                clockSequence = getRandomSeq();
            }
        }
        else if (version === UUIDVersions_1.UUIDVersions.Three || version === UUIDVersions_1.UUIDVersions.Five) {
            if (!hash) {
                throw new Error(strings_1.strings.HASH_ARGUMENT_MISSING);
            }
            let clockSequenceStr = '';
            clockSequenceStr += hash.slice(16, 18);
            clockSequenceStr += hash.slice(18, 20);
            const clockSequenceBinStr = parseInt(clockSequenceStr, 16).toString(2).padStart(14, '0');
            clockSequence = convertBinStrToUint8Array_1.convertBinStrToUint8Array(clockSequenceBinStr);
        }
        else {
            clockSequence = new Uint8Array([0, 0,]);
        }
        return clockSequence;
    };
    exports.default = exports.clockSequenceGetter;
});
