(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "crypto", "./isNode", "./strings", "./uintArrayAsNumber"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const crypto_1 = require("crypto");
    const isNode_1 = require("./isNode");
    const strings_1 = require("./strings");
    const uintArrayAsNumber_1 = require("./uintArrayAsNumber");
    exports.createHash = (() => {
        if (isNode_1.isNode()) {
            return (algorithm) => {
                const realHasher = crypto_1.createHash(algorithm);
                return {
                    update(payload) {
                        realHasher.update(payload);
                    },
                    digest() {
                        return new Promise((resolve) => {
                            resolve(realHasher.digest('hex'));
                        });
                    },
                };
            };
        }
        else {
            const getBufferFromText = (str) => {
                if (window.TextEncoder) {
                    return new TextEncoder('utf-8').encode(str);
                }
                var utf8 = unescape(encodeURIComponent(str));
                var result = new Uint8Array(utf8.length);
                for (let ii = 0; ii < utf8.length; ii++) {
                    result[ii] = utf8.charCodeAt(ii);
                }
                return result;
            };
            if (window && 'msCrypto' in window) {
                return (algorithm) => {
                    if (algorithm === 'md5') {
                        throw new Error(strings_1.strings.MD5_IN_BROWSER);
                    }
                    const hasher = {
                        __payload: null,
                        update(payload) {
                            hasher.__payload = payload;
                        },
                        digest() {
                            const buffer = getBufferFromText(str);
                            return new Promise((resolve) => {
                                const result = window.msCrypto.subtle.digest('SHA-256', buffer);
                                console.log(result);
                                const asNumber = uintArrayAsNumber_1.uintArrayAsNumber(result);
                                console.log(asNumber);
                                const asHex = asNumber.toString(16);
                                console.log(asHex);
                                resolve(asHex);
                            });
                        },
                    };
                    return hasher;
                };
            }
            else if (window && 'crypto' in window) {
                return (algorithm) => {
                    if (algorithm === 'md5') {
                        throw new Error(strings_1.strings.MD5_IN_BROWSER);
                    }
                    const hasher = {
                        __payload: '',
                        update(payload) {
                            hasher.__payload = payload;
                        },
                        digest() {
                            const buffer = getBufferFromText(hasher.__payload);
                            return new Promise((resolve) => {
                                window.crypto.subtle.digest('SHA-256', buffer).then((result) => {
                                    console.log(result);
                                    const asNumber = uintArrayAsNumber_1.uintArrayAsNumber(result);
                                    console.log(asNumber);
                                    const asHex = asNumber.toString(16);
                                    console.log(asHex);
                                    resolve(asHex);
                                });
                            });
                        },
                    };
                    return hasher;
                };
            }
            else {
                return () => {
                    throw new Error(strings_1.strings.NO_HASHER_FOUND);
                };
            }
        }
    })();
});
