(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./convertBinStrToUint8Array", "./getMAC", "./getLastResults", "./randomBytesGenerator", "./strings", "./Enums/UUIDVersions"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const convertBinStrToUint8Array_1 = require("./convertBinStrToUint8Array");
    const getMAC_1 = require("./getMAC");
    const getLastResults_1 = require("./getLastResults");
    const randomBytesGenerator_1 = require("./randomBytesGenerator");
    const strings_1 = require("./strings");
    const UUIDVersions_1 = require("./Enums/UUIDVersions");
    function nodeIdentifierGetter(version, hash) {
        let nodeIdentifier;
        if (version === UUIDVersions_1.UUIDVersions.One) {
            const lastResults = getLastResults_1.getLastResults();
            if (lastResults &&
                lastResults.nodeIdentifier &&
                'BYTES_PER_ELEMENT' in lastResults.nodeIdentifier) {
                return lastResults.nodeIdentifier;
            }
            nodeIdentifier = getMAC_1.getMAC();
        }
        else if (version === UUIDVersions_1.UUIDVersions.Three || version === UUIDVersions_1.UUIDVersions.Five) {
            if (!hash) {
                throw new Error(strings_1.strings.HASH_ARGUMENT_MISSING);
            }
            let nodeIdentifierStr = '';
            nodeIdentifierStr += hash.slice(20, 32);
            let nodeIdentifierBinStr = parseInt(nodeIdentifierStr, 16)
                .toString(2)
                .padStart(48, '0');
            nodeIdentifierBinStr =
                nodeIdentifierBinStr.slice(0, 7) +
                    '1' +
                    nodeIdentifierBinStr.slice(8);
            nodeIdentifier = convertBinStrToUint8Array_1.convertBinStrToUint8Array(nodeIdentifierBinStr);
        }
        else if (version === UUIDVersions_1.UUIDVersions.Four) {
            nodeIdentifier = randomBytesGenerator_1.randomBytesGenerator(6);
        }
        else if (version === UUIDVersions_1.UUIDVersions.Nil) {
            nodeIdentifier = new Uint8Array([0, 0, 0, 0, 0, 0,]);
        }
        else {
            throw new Error(strings_1.strings.UUID_VERSION_INVALID);
        }
        return nodeIdentifier;
    }
    exports.nodeIdentifierGetter = nodeIdentifierGetter;
    exports.default = nodeIdentifierGetter;
});
