var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "big-integer", "./convertBinStrToUint8Array", "./getHundredsOfNanosecondsSinceGregorianReform", "./TypeGuards/isUUIDVersion", "./randomBytesGenerator", "./strings", "./Enums/UUIDVersions"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const big_integer_1 = __importDefault(require("big-integer"));
    const convertBinStrToUint8Array_1 = require("./convertBinStrToUint8Array");
    const getHundredsOfNanosecondsSinceGregorianReform_1 = require("./getHundredsOfNanosecondsSinceGregorianReform");
    const isUUIDVersion_1 = require("./TypeGuards/isUUIDVersion");
    const randomBytesGenerator_1 = require("./randomBytesGenerator");
    const strings_1 = require("./strings");
    const UUIDVersions_1 = require("./Enums/UUIDVersions");
    exports.timestampGetter = (version, hash) => {
        if (!isUUIDVersion_1.isUUIDVersion(version)) {
            throw new Error(strings_1.strings.UUID_VERSION_INVALID);
        }
        let timestamp;
        if (version === UUIDVersions_1.UUIDVersions.One) {
            const currentTimestamp = getHundredsOfNanosecondsSinceGregorianReform_1.getHundredsOfNanosecondsSinceGregorianReform();
            const timestampStr = currentTimestamp.toString(2);
            timestamp = convertBinStrToUint8Array_1.convertBinStrToUint8Array(timestampStr);
        }
        else if (version === UUIDVersions_1.UUIDVersions.Three || version === UUIDVersions_1.UUIDVersions.Five) {
            if (!hash) {
                throw new Error(strings_1.strings.HASH_ARGUMENT_MISSING);
            }
            let timestampStr = '';
            timestampStr = hash.slice(0, 8);
            timestampStr = hash.slice(8, 12) + timestampStr;
            timestampStr = hash.slice(12, 16) + timestampStr;
            console.log(timestampStr);
            const timestampBinStr = big_integer_1.default(timestampStr, 16).toString(2);
            timestamp = convertBinStrToUint8Array_1.convertBinStrToUint8Array(timestampBinStr);
        }
        else if (version === UUIDVersions_1.UUIDVersions.Four) {
            timestamp = randomBytesGenerator_1.randomBytesGenerator(8);
        }
        else {
            timestamp = new Uint8Array([0, 0, 0, 0, 0, 0, 0, 0,]);
        }
        timestamp[0] = Math.min(32, timestamp[0]);
        timestamp = new Uint8Array('0'.repeat(8 - timestamp.length)
            .split('')
            .map(parseInt)
            .concat(Array.prototype.slice.call(timestamp)));
        return timestamp;
    };
    exports.default = exports.timestampGetter;
});
