import bigInt from 'big-integer';
import { convertBinStrToUint8Array, } from './convertBinStrToUint8Array';
import { getHundredsOfNanosecondsSinceGregorianReform, } from './getHundredsOfNanosecondsSinceGregorianReform';
import { isUUIDVersion, } from './TypeGuards/isUUIDVersion';
import { randomBytesGenerator, } from './randomBytesGenerator';
import { strings, } from './strings';
import { UUIDVersions, } from './Enums/UUIDVersions';
export const timestampGetter = (version, hash) => {
    if (!isUUIDVersion(version)) {
        throw new Error(strings.UUID_VERSION_INVALID);
    }
    let timestamp;
    if (version === UUIDVersions.One) {
        const currentTimestamp = getHundredsOfNanosecondsSinceGregorianReform();
        const timestampStr = currentTimestamp.toString(2);
        timestamp = convertBinStrToUint8Array(timestampStr);
    }
    else if (version === UUIDVersions.Three || version === UUIDVersions.Five) {
        if (!hash) {
            throw new Error(strings.HASH_ARGUMENT_MISSING);
        }
        let timestampStr = '';
        timestampStr = hash.slice(0, 8);
        timestampStr = hash.slice(8, 12) + timestampStr;
        timestampStr = hash.slice(12, 16) + timestampStr;
        console.log(timestampStr);
        const timestampBinStr = bigInt(timestampStr, 16).toString(2);
        timestamp = convertBinStrToUint8Array(timestampBinStr);
    }
    else if (version === UUIDVersions.Four) {
        timestamp = randomBytesGenerator(8);
    }
    else {
        timestamp = new Uint8Array([0, 0, 0, 0, 0, 0, 0, 0,]);
    }
    timestamp[0] = Math.min(32, timestamp[0]);
    timestamp = new Uint8Array('0'.repeat(8 - timestamp.length)
        .split('')
        .map(parseInt)
        .concat(Array.prototype.slice.call(timestamp)));
    return timestamp;
};
export default timestampGetter;
